@@ -61,7 +61,7 @@
 #include "sudo_auth.h"
 
 #ifndef lint
-static const char rcsid[] = "$Sudo: passwd.c,v 1.9 2001/12/14 19:52:53 millert Exp $";
+static const char rcsid[] = "$Sudo: passwd.c,v 1.10 2002/01/17 05:24:28 millert Exp $";
 #endif /* lint */
 
 #define DESLEN			13
@@ -74,7 +74,7 @@ passwd_init(pw, promptp, auth)
     sudo_auth *auth;
 {
 #ifdef HAVE_SKEYACCESS
-    if (skeyaccess(pw->pw_name, user_tty, NULL, NULL) == 0)
+    if (skeyaccess(pw, user_tty, NULL, NULL) == 0)
 	return(AUTH_FATAL);
 #endif
     return(AUTH_SUCCESS);
