@@ -98,7 +98,7 @@
 #include "version.h"
 
 #ifndef lint
-static const char rcsid[] = "$Sudo: sudo.c,v 1.314 2002/01/13 18:29:23 millert Exp $";
+static const char rcsid[] = "$Sudo: sudo.c,v 1.318 2002/01/15 23:43:59 millert Exp $";
 #endif /* lint */
 
 /*
@@ -260,7 +260,7 @@ main(argc, argv, envp)
      * set the real, effective and saved uids to 0 and use set_perms_fallback()
      * instead of set_perms_posix().
      */
-#if defined(_SC_SAVED_IDS) && defined(_SC_VERSION)
+#if !defined(NO_SAVED_IDS) && defined(_SC_SAVED_IDS) && defined(_SC_VERSION)
     if (!def_flag(I_STAY_SETUID) && set_perms == set_perms_posix) {
 	if (setuid(0)) {
 	    perror("setuid(0)");
@@ -287,9 +287,6 @@ main(argc, argv, envp)
 	    log_error(NO_MAIL|MSG_ONLY, "no passwd entry for %s!", *user_runas);
     }
 
-    /* Build up custom environment that avoids any nasty bits. */
-    new_environ = rebuild_env(sudo_mode, envp);
-
     /* This goes after the sudoers parse since we honor sudoers options. */
     if (sudo_mode == MODE_KILL || sudo_mode == MODE_INVALIDATE) {
 	remove_timestamp((sudo_mode == MODE_KILL));
@@ -315,9 +312,9 @@ main(argc, argv, envp)
     if ((sudo_mode & MODE_IMPLIED_SHELL) && !def_flag(I_SHELL_NOARGS))
 	usage(1);
 
-    /* May need to set $HOME to target user. */
-    if (def_flag(I_ALWAYS_SET_HOME) ||
-	((sudo_mode & MODE_SHELL) && def_flag(I_SET_HOME)))
+    /* May need to set $HOME to target user if we are running a command. */
+    if ((sudo_mode & MODE_RUN) && (def_flag(I_ALWAYS_SET_HOME) ||
+	((sudo_mode & MODE_SHELL) && def_flag(I_SET_HOME))))
 	sudo_mode |= MODE_RESET_HOME;
 
     /* Bail if a tty is required and we don't have one.  */
@@ -335,6 +332,9 @@ main(argc, argv, envp)
     if (!(validated & FLAG_NOPASS))
 	check_user();
 
+    /* Build up custom environment that avoids any nasty bits. */
+    new_environ = rebuild_env(sudo_mode, envp);
+
     if (validated & VALIDATE_OK) {
 	/* Finally tell the user if the command did not exist. */
 	if (cmnd_status == NOT_FOUND_DOT) {
@@ -511,7 +511,8 @@ init_vars(sudo_mode)
 	log_error(0, "uid %ld does not exist in the passwd file!",
 	    (long) pw.pw_uid);
     }
-    user_shell = sudo_user.pw->pw_shell;
+    if (user_shell == NULL || *user_shell == '\0')
+	user_shell = sudo_user.pw->pw_shell;
 
     /* It is now safe to use log_error() and set_perms() */
 
@@ -888,7 +889,7 @@ initial_setup()
     (void) sigaction(SIGCHLD, &sa, NULL);
 
     /* Set set_perms pointer to the correct function */
-#if defined(_SC_SAVED_IDS) && defined(_SC_VERSION)
+#if !defined(NO_SAVED_IDS) && defined(_SC_SAVED_IDS) && defined(_SC_VERSION)
     if (sysconf(_SC_SAVED_IDS) == 1 && sysconf(_SC_VERSION) >= 199009)
 	set_perms = set_perms_posix;
     else
@@ -950,17 +951,15 @@ set_fqdn()
     struct hostent *hp;
     char *p;
 
-    if (def_flag(I_FQDN)) {
-	if (!(hp = gethostbyname(user_host))) {
-	    log_error(MSG_ONLY|NO_EXIT,
-		"unable to lookup %s via gethostbyname()", user_host);
-	} else {
-	    free(user_host);
-	    user_host = estrdup(hp->h_name);
-	}
+    if (!(hp = gethostbyname(user_host))) {
+	log_error(MSG_ONLY|NO_EXIT,
+	    "unable to lookup %s via gethostbyname()", user_host);
+    } else {
+	if (user_shost != user_host)
+	    free(user_shost);
+	free(user_host);
+	user_host = estrdup(hp->h_name);
     }
-    if (user_shost != user_host)
-	free(user_shost);
     if ((p = strchr(user_host, '.'))) {
 	*p = '\0';
 	user_shost = estrdup(user_host);
